package GrowthForecast;

use strict;
use warnings;
use utf8;

our $VERSION = '0.80';

1;

__END__

=head1 NAME

GrowthForecast - Lightning Fast Graphing/Visualization

=head1 SYNOPSIS

  % growthforecast.pl -h

=head1 DESCRIPTION

GrowthForecast is graphing/visualization web tool built on RRDtool
See growthforecast.pl perldoc.

=head1 AUTHOR

Masahiro Nagano E<lt>kazeburo {at} gmail.comE<gt>

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

